'部材３次元体解析V65の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module Bars3Com
    'トラス、ラーメン、トラス・ラーメン混合の３次元弾性解析
    'トラス・・・節点が回転自由
    'ラーメン・・・節点が固定接続
    'トラス・ラーメン混合・・・節点が回転自由か固定接続か任意
    '２節点の均等断面棒要素を使用したＦＥＭ解析
    '要素断面はYZ各軸回りの断面２次モーメントも使用

    'V56での改良点・・・X3Dファイル出力の文字コードをShift-JISからUTF8に変更
    'V57での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V58での改良点・・・３次元グラフィック図で回転操作後の部分拡大の角度を保持
    'V59での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V60での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V61での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V62での改良点・・・構造図と応力図で部材の線分幅を縮小、
    '                   構造図画面で条件設定部の表示を２区分から３区分に改善
    'V63での改良点・・・拘束条件で座標軸方向の欠けエラーをチェック
    'V64での改良点・・・入力データファイルで拘束節点の形式を改善
    'V65での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$         '計算結果表示データ
    Public Cr$              'CR+LFのコード
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fm$              '入力データのファイル名
    Public Fy$              '計算結果のファイル名
    Public Title$           'タイトル

    Public Timst As Date          '計算開始時刻

    Public InitDraw As Integer      '作画の初期化の有無（0:なし、1:有り）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public KindDPL As Integer       '変形区分（0:小変形、1:大変形、2:後変形）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindGrav As Integer      '自重計算の有無（0:なし、1:有り）
    Public KindPost As Integer      '後処理の種類（1:変位図、2:応力図、3:端数処理、
    '                                4:主要値表示）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:拘束節点、
    '                                3:節点荷重、4:表面分布荷重、5:バネ支持、
    '                                6:要素の削除、7:要素の追加）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public Mprop As Integer         '材料データの成分数(=14)
    Public MultiColor As Integer    '材料別色分け（0:なし、1:有り）
    Public Nall As Integer          '変数の総数
    Public Nbound, Nboundb As Integer        '拘束節点数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(=6)
    Public NEdraw As Integer        '作画要素の数
    Public Nelem As Integer         '要素数
    Public Nforce, Nforceb As Integer        '荷重節点数
    Public NKmax As Integer         '最大繰り返し計算回数
    Public Nmat As Integer          '材料データ数
    Public Npoint As Integer        '節点数
    Public Nspring, Nspringb As Integer       'バネ支持個数
    Public Nsurf, Nsurfb As Integer         '表面分布荷重数
    Public Ntype As Integer         '問題のタイプ（1:トラス、2:ラーメン、3:両者混合）
    Public Nvar As Integer          '１要素当りの変数の総数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public DispRatio As Single      '変位作画の倍率
    Public EpsCalc As Single        '収束基準誤差率
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public ArrText As New ArrayList()

    Public UFileA(6) As String      '最近使用したﾃﾞｰﾀﾌｧｲﾙの保存配列

    Public Jdirect(,) As Integer    '節点拘束の各変位方向拘束有無(0:非拘束、1:拘束）
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KEorder() As Integer     '作画順の要素番号
    Public KEsurf() As Integer      '表面分布荷重の要素番号
    Public KindElem() As Integer    '要素種類（1:トラス、2:ラーメン、3:左節点回転自由、
    '                                4:右節点回転自由）
    Public KPforce() As Integer     '節点荷重の節点番号
    Public KPspring(,) As Integer   'バネ支持の節点番号
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public LrD() As Integer         '元ﾃﾞｰﾀ節点の第1変数の元全体ﾏﾄﾘｯｸｽ番号
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '拘束の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号
    Public NvarPD() As Integer      '元ﾃﾞｰﾀ節点の変数の数

    Public AngleYBar() As Single    '断面Y軸の角度（最上向きから左回り、°）
    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public CenterPos(3) As Single   '構造中心の座標
    Public CosYBar(,) As Single     '部材Y軸の全体X,Y,Z軸に対する方向余弦
    Public Cspring(,) As Single     'バネ定数
    Public DataMat(,) As Single     '各材料の物性値
    Public Force(,) As Single       '節点荷重値（各軸方向）
    Public Fspring(,) As Single     '初期バネ力（各軸方向）
    Public GravMulti(3) As Single   '自重のX,Y,Z方向の倍率
    Public PnormYB() As Single      '部材Y座標軸方向の垂直分布荷重（単位長さ当り）
    Public PnormZB() As Single      '部材Z座標軸方向の垂直分布荷重（単位長さ当り）
    Public Ptang() As Single        '部材の接線方向分布荷重（単位長さ当り）
    Public Tdisp(,) As Single       '結果の変位
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public Tu() As Single           'FEM計算の解
    Public Vdirect(,) As Single     '節点拘束の各軸方向拘束値
    Public Xdata(), Ydata(), Zdata() As Single  '節点座標
    Public XE(2), YE(2), ZE(2) As Single    '要素計算の配列
    Public Xmodel(), Ymodel(), Zmodel() As Single   'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xnew(), Ynew(), Znew() As Single   '変位入りの節点座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub AdjustRatio(ByRef ratio As Single)
        '変位比率の調整
        Dim nb As Integer
        Dim b1, b2, std As Single

        std = 0.05
        b1 = std / ratio
        If b1 < 1.0 Then
            ratio = 1.0
        ElseIf b1 < 10.0 Then
            b2 = b1 * 10
            ratio = Int(b2) / 10.0
        Else    '上位2桁を採用
            b2 = b1
            nb = 0
            Do
                b2 = b2 / 10
                nb = nb + 1
                If b2 < 10 Then
                    Exit Do
                End If
            Loop
            b2 = Int(b2 * 10)
            If nb < 2 Then
                ratio = b2
            Else
                ratio = b2 * 10 ^ (nb - 1)
            End If
        End If

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
        ByRef rlenmaxb As Single, ByRef xd() As Single, _
        ByRef yd() As Single, ByRef zd() As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmaxb･･･1軸の最大長さ
        'xd(),yd(),zd()･･･節点の座標
        Dim ib, ie, ii, ik, j As Integer
        Dim b1 As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = xd(ib)
                        AxisMin(2) = yd(ib)
                        AxisMin(3) = zd(ib)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = xd(ib)
                                Case 2
                                    b1 = yd(ib)
                                Case 3
                                    b1 = zd(ib)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmaxb = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmaxb Then
                rlenmaxb = b1
            End If
        Next ii

    End Sub

    Sub AxisCheck2(ByVal k1 As Integer, ByVal k2 As Integer, _
        ByRef xmin As Single, ByRef xmax As Single, _
        ByRef ymin As Single, ByRef ymax As Single)
        'ｽｸﾘｰﾝにおけるX、Y軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim ib, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = Xscreen(ib) : xmax = xmin
                        ymin = Yscreen(ib) : ymax = ymin
                    Else
                        If Xscreen(ib) < xmin Then
                            xmin = Xscreen(ib)
                        ElseIf Xscreen(ib) > xmax Then
                            xmax = Xscreen(ib)
                        End If
                        If Yscreen(ib) < ymin Then
                            ymin = Yscreen(ib)
                        ElseIf Yscreen(ib) > ymax Then
                            ymax = Yscreen(ib)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisCheckM(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef axmax() As Single, ByRef axmin() As Single)
        'ﾓﾃﾞﾙ座標のX、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'axmax()･･･X,Y,Z座標の最大値
        'axmin()･･･X,Y,Z座標の最小値
        Dim ie, ii, ik, j, k As Integer
        Dim b1 As Single

        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        axmin(1) = Xmodel(k)
                        axmin(2) = Ymodel(k)
                        axmin(3) = Zmodel(k)
                        For ii = 1 To 3
                            axmax(ii) = axmin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xmodel(k)
                                Case 2
                                    b1 = Ymodel(k)
                                Case 3
                                    b1 = Zmodel(k)
                            End Select
                            If b1 < axmin(ii) Then
                                axmin(ii) = b1
                            ElseIf b1 > axmax(ii) Then
                                axmax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = axmax(1) - axmin(1)
        For ii = 2 To 3
            b1 = axmax(ii) - axmin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheckS(ByVal k1 As Integer, ByRef k2 As Integer, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xdb(),ydb()･･･節点座標
        Dim i, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = xdb(i) : xmax = xmin
                        ymin = ydb(i) : ymax = ymin
                    Else
                        If xdb(i) < xmin Then
                            xmin = xdb(i)
                        ElseIf xdb(i) > xmax Then
                            xmax = xdb(i)
                        End If
                        If ydb(i) < ymin Then
                            ymin = ydb(i)
                        ElseIf ydb(i) > ymax Then
                            ymax = ydb(i)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisSet(ByRef tdb(,) As Single)
        '座標の設定
        Dim i As Integer
        For i = 1 To Npoint
            Xnew(i) = Xdata(i) + DispRatio * tdb(i, 1)
            Ynew(i) = Ydata(i) + DispRatio * tdb(i, 2)
            Znew(i) = Zdata(i) + DispRatio * tdb(i, 3)
        Next i
    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb, Xdata, Ydata, Zdata)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub CfileDispU(ByVal fxx$, ByRef tfbuf(,) As Single)
        '計算結果ﾌｧｲﾙから変位ﾃﾞｰﾀの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        'tfbuf(,)･･･節点の各座標方向の変位
        Dim aa$
        Dim ip, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Do
            aa$ = sr.ReadLine
        Loop Until aa$ = "＊＊ 変位等の出力 ＊＊"
        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To 3
                tfbuf(ip, j) = dat(j)
            Next j
        Next ip

        sr.Close()

    End Sub

    Sub ColorSet(ByVal kb As Integer, ByRef colorx As Color)
        '色の設定
        'kb･･･指定番号（1～）
        'colorx･･･色
        Dim kcount As Integer
        kcount = (kb - 1) Mod 24
        Select Case kcount
            Case 0
                colorx = Color.Black
            Case 1
                colorx = Color.Green
            Case 2
                colorx = Color.DarkOrange
            Case 3
                colorx = Color.FromArgb(0, 0, 128)
            Case 4
                colorx = Color.FromArgb(128, 0, 0)
            Case 5
                colorx = Color.FromArgb(192, 99, 0)
            Case 6
                colorx = Color.Violet
            Case 7
                colorx = Color.DarkBlue
            Case 8
                colorx = Color.FromArgb(90, 90, 90)
            Case 9
                colorx = Color.DarkGreen
            Case 10
                colorx = Color.DarkRed
            Case 11
                colorx = Color.Brown
            Case 12
                colorx = Color.FromArgb(0, 128, 99)
            Case 13
                colorx = Color.FromArgb(192, 99, 99)
            Case 14
                colorx = Color.LimeGreen
            Case 15
                colorx = Color.Purple
            Case 16
                colorx = Color.Gray
            Case 17
                colorx = Color.FromArgb(99, 99, 192)
            Case 18
                colorx = Color.FromArgb(160, 128, 0)
            Case 19
                colorx = Color.FromArgb(160, 128, 99)
            Case 20
                colorx = Color.BurlyWood
            Case 21
                colorx = Color.DarkSeaGreen
            Case 22
                colorx = Color.DarkKhaki
            Case 23
                colorx = Color.DarkOrchid
        End Select

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
    ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
    ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim a2$, aa$, aerr1$
        Dim i, ib, ic, icc, id, ie, im, ip, j As Integer
        Dim k, n1, n2, nb As Integer
        Dim b1, b2, b3 As Single
        Dim abuf$(10)

        a2$ = ""
        aerr1$ = "になっています！"
        num = 0

        If Npoint < 2 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nbound < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  拘束節点数が" + Str$(Nbound) + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            n1 = NNelem(ie, 1)
            n2 = NNelem(ie, 2)
            If n1 = n2 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " ２つの節点番号が" + Str$(n1) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            ic = 0
            For k = 1 To 2
                If k = 1 Then
                    nb = n1
                Else
                    nb = n2
                End If
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                    abuf$(num) = aa$ + "節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                Else    '節点接続のﾁｪｯｸ
                    If ic = 0 And Nelem > 1 Then
                        For j = 1 To Nelem
                            If j <> ie Then
                                If nb = NNelem(j, 1) Or nb = NNelem(j, 2) Then
                                    ic = 1
                                    Exit For
                                End If
                            End If
                        Next j
                    End If
                End If
            Next k
            If ic = 0 And Nelem > 1 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf(num) = aa$ + " 節点が接続されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If Ntype = 3 Then
                If KindElem(ie) < 1 Or KindElem(ie) > 4 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                    abuf(num) = aa$ + " 要素種類の番号が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            End If
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = Xdata(ip) - Xdata(j)
                b2 = Ydata(ip) - Ydata(j)
                b3 = Zdata(ip) - Zdata(j)
                If b1 = 0 And b2 = 0 And b3 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf$(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '拘束ﾃﾞｰﾀのﾁｪｯｸ
        Dim kdir(6) As Integer
        For ib = 1 To Nbound
            nb = Nrest(ib)
            If nb < 1 Or nb > Npoint Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  拘束番号" + Str$(ib) + "で"
                abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = ib + 1 To Nbound
                If nb = Nrest(j) Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  拘束番号" + Str$(ib) + "、" + Str$(j) + "で"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "が重複しています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
            ic = 0
            For id = 1 To Ndim
                If Jdirect(ib, id) <> 0 Then
                    ic = 1
                    kdir(id) = 1
                End If
            Next id
            If ic = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  拘束番号" + Str$(ib) + "で"
                abuf$(num) = aa$ + " 拘束がありません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ib

        ic = 1
        k = 0
        For j = 1 To 3
            If kdir(j) = 0 Then
                ic = 0
                k = k + 1
            End If
        Next j
        If ic = 0 Then
            Select Case k
                Case 1
                    For j = 1 To 3
                        If kdir(j) = 0 Then
                            Select Case j
                                Case 1
                                    a2$ = "X方向が拘束されていません！"
                                Case 2
                                    a2$ = "Y方向が拘束されていません！"
                                Case 3
                                    a2$ = "Z方向の拘束がありません！"
                            End Select
                            Exit For
                        End If
                    Next j
                Case 2
                    For j = 1 To 3
                        If kdir(j) > 0 Then
                            Select Case j
                                Case 1
                                    a2$ = "Y,Z方向が拘束されていません！"
                                Case 2
                                    a2$ = "X,Z方向が拘束されていません！"
                                Case 3
                                    a2$ = "X,Y方向が拘束されていません！"
                            End Select
                            Exit For
                        End If
                    Next j
                Case 3
                    a2$ = "X,Y,Zの全方向が拘束されていません！"
            End Select
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  " + a2$
            If num = 10 Then
                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                Exit Sub
            End If
        End If

        If Ntype = 1 Then
            If j < 5 And j <> 3 Then
                ic = 1
            ElseIf j = 12 Or j = 13 Then
                ic = 1
            End If
        ElseIf j <> 3 Then
            ic = 1
        End If

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For im = 1 To Nmat
            For j = 1 To Mprop
                b1 = DataMat(im, j)
                If Ntype = 1 Then
                    If j < 5 Then
                        ic = 1
                    ElseIf j = 12 Or j = 13 Then
                        ic = 1
                    Else
                        ic = 0
                    End If
                Else
                    ic = 1
                End If
                If b1 <= 0 And ic = 1 Then
                    icc = 0
                    Select Case j
                        Case 1
                            a2$ = " ヤング率が"
                        Case 2
                            a2$ = " ポアソン比が"
                        Case 3
                            If KindGrav > 0 Then
                                a2$ = " 比重量が"
                            Else
                                icc = 1
                            End If
                        Case 4
                            a2$ = " 面積が"
                        Case 5
                            a2$ = " ＩyBが"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 6
                            a2$ = " ＩzBが"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 7
                            a2$ = " 捩り係数Kが"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 8
                            a2$ = " Ｚy1が"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 9
                            a2$ = " Ｚy2が"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 10
                            a2$ = " Ｚz1が"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 11
                            a2$ = " Ｚz2が"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                        Case 12
                            a2$ = " Ｃsyが"
                        Case 13
                            a2$ = " Ｃszが"
                        Case 14
                            a2$ = " Ｚpが"
                            If Ntype = 1 Then
                                icc = 1
                            End If
                    End Select
                    If icc = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                        abuf$(num) = aa$ + a2$ + Str$(b1) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                End If
            Next j
        Next im

        If Nforce > 0 Then  '荷重節点のﾁｪｯｸ
            For ib = 1 To Nforce
                nb = KPforce(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  荷重節点の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nsurf > 0 Then   '表面分布荷重のﾁｪｯｸ
            For ib = 1 To Nsurf
                nb = KEsurf(ib)
                If nb < 1 Or nb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 要素番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If PnormYB(ib) = 0 And PnormZB(ib) = 0 And Ptang(ib) = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  表面分布荷重の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 荷重がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nspring > 0 Then   'ﾊﾞﾈ支持節点のﾁｪｯｸ
            For ib = 1 To Nspring
                For j = 1 To 2
                    nb = KPspring(ib, j)
                    If j = 1 Then
                        If nb < 1 Or nb > Npoint Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    ElseIf nb > 0 Then
                        If nb < 1 Or nb > Npoint Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号2が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        ElseIf nb = KPspring(ib, 1) Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  バネ支持No." + Str$(ib) + "で"
                            abuf$(num) = aa$ + " 節点番号1、2共" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    End If
                Next j
                ic = 0
                For j = 1 To 3
                    If Cspring(ib, j) <> 0 Then
                        ic = 1
                        Exit For
                    End If
                Next j
                If ic = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  バネNo." + Str$(ib) + "で"
                    abuf$(num) = aa$ + " バネ定数がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If KindGrav > 0 Then    '自重倍率のﾁｪｯｸ
            ic = 0
            For i = 1 To 3
                If GravMulti(i) <> 0 Then
                    ic = 1
                    Exit For
                End If
            Next i
            If ic = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  自重のX,Y,Z軸方向の倍率が設定されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, id, ie, im, ip, j, jb As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Bars3D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nforce = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nsurf = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nspring = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindGrav = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)

        ReDim KindElem(Nelem), NNelem(Nelem, 2), NumMat(Nelem), CosYBar(Nelem, 3)
        ReDim AngleYBar(Nelem)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)
        ReDim Nrest(Nbound), Jdirect(Nbound, 6), Vdirect(Nbound, 6)
        ReDim DataMat(Nmat, Mprop)
        ReDim KPforce(Nforce), Force(Nforce, 6)
        ReDim KEsurf(Nsurf)
        ReDim PnormYB(Nsurf), PnormZB(Nsurf), Ptang(Nsurf)
        ReDim KPspring(Nspring, 2), Cspring(Nspring, 3), Fspring(Nspring, 3)


        aa$ = sr.ReadLine
        If Ntype < 3 Then
            For ie = 1 To Nelem
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NumMat(ie) = dat(1)
                For j = 1 To 2
                    NNelem(ie, j) = dat(j + 1)
                Next j
                AngleYBar(ie) = dat(4)
            Next ie
        Else
            For ie = 1 To Nelem
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KindElem(ie) = dat(1)
                NumMat(ie) = dat(2)
                For j = 1 To 2
                    NNelem(ie, j) = dat(j + 2)
                Next j
                AngleYBar(ie) = dat(5)
            Next ie
        End If

        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(ip) = dat(1)
            Ydata(ip) = dat(2)
            Zdata(ip) = dat(3)
        Next ip

        If Nbound > 0 Then
            aa$ = sr.ReadLine
            If Len(aa$) < 10 Then
                For i = 1 To Nbound
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    Nrest(i) = dat(3)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To 3
                        jb = 4 * (j - 1)
                        Jdirect(i, j) = dat(jb + 1)
                        Vdirect(i, j) = dat(jb + 3)
                    Next j
                    If Ndim > 3 Then
                        aa$ = sr.ReadLine
                        dat = aa.Split(",")
                        For j = 4 To Ndim
                            jb = 4 * (j - 4)
                            Jdirect(i, j) = dat(jb + 1)
                            Vdirect(i, j) = dat(jb + 3)
                        Next j
                    End If
                Next i
            Else
                For i = 1 To Nbound
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    Nrest(i) = dat(1)
                    For j = 1 To Ndim
                        jb = 2 * j
                        Jdirect(i, j) = dat(jb)
                        Vdirect(i, j) = dat(jb + 1)
                    Next j
                Next i
            End If
        End If

        aa$ = sr.ReadLine
        For im = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To Mprop
                DataMat(im, j) = dat(j)
            Next j
        Next im

        If Nforce > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Nforce
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPforce(ib) = dat(1)
                For id = 1 To Ndim
                    Force(ib, id) = dat(id + 1)
                Next id
            Next ib
        End If

        If Nsurf > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Nsurf
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KEsurf(ib) = dat(1)
                PnormYB(ib) = dat(2)
                PnormZB(ib) = dat(3)
                Ptang(ib) = dat(4)
            Next ib
        End If

        If Nspring > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nspring
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To 2
                    KPspring(i, j) = dat(j)
                Next j
                For j = 1 To 3
                    jb = 2 * j
                    Cspring(i, j) = dat(jb + 1)
                    Fspring(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If KindGrav > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For i = 1 To 3
                GravMulti(i) = dat(i - 1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, id, ie, im, ip, j, j1, j2, j3 As Integer
        Dim b1, b2, b3 As Single
        Dim ad1(6), ad2(6) As String

        Dim sw As New IO.StreamWriter(fxx$)

        aa$ = "プログラム名" + "," + "Bars3D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "問題のタイプ" + "," + Str(Ntype)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "拘束節点数" + "," + Str(Nbound)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "荷重節点数" + "," + Str(Nforce)
        sw.WriteLine(aa$)
        aa$ = "分布荷重個数" + "," + Str(Nsurf)
        sw.WriteLine(aa$)
        aa$ = "バネ支持個数" + "," + Str(Nspring)
        sw.WriteLine(aa$)
        aa$ = "自重の有無（0:なし、1:あり）" + "," + Str(KindGrav)
        sw.WriteLine(aa$)
        aa$ = "１節点当りの変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)

        If Ntype < 3 Then
            sw.WriteLine("要素データ (要素番号、材料番号、2節点番号、断面Y軸の角度)")
            For ie = 1 To Nelem
                aa$ = Str$(ie) + ",  " + Str$(NumMat(ie))
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(NNelem(ie, j))
                Next j
                aa$ = aa$ + ",  " + Str$(AngleYBar(ie))
                sw.WriteLine(aa$)
            Next ie
        Else
            sw.WriteLine("要素データ (要素番号、要素種類、材料番号、2節点番号、断面Y軸の角度)")
            For ie = 1 To Nelem
                aa$ = Str$(ie) + ",  " + Str$(KindElem(ie))
                aa$ = aa$ + ",  " + Str$(NumMat(ie))
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(NNelem(ie, j))
                Next j
                aa$ = aa$ + ",  " + Str$(AngleYBar(ie))
                sw.WriteLine(aa$)
            Next ie
        End If

        aa$ = "節点データ（節点番号、X、Y、Z）"
        sw.WriteLine(aa$)
        For ip = 1 To Npoint
            aa$ = Str$(ip) + ",  " + Str$(Xdata(ip))
            aa$ = aa$ + ",  " + Str$(Ydata(ip)) + ",  " + Str$(Zdata(ip))
            sw.WriteLine(aa$)
        Next ip

        If Nbound > 0 Then
            aa$ = "拘束節点データ（拘束No.、節点番号、X拘束・値、Y拘束・値、Z拘束・値"
            If Ntype = 1 Then
                aa$ = aa$ + "）"
            Else
                aa$ = aa$ + "θx拘束・値、θy拘束・値、θz拘束・値）"
            End If
            sw.WriteLine(aa$)
            For i = 1 To Nbound
                aa$ = Str(i) + "," + Str(Nrest(i))
                For j = 1 To Ndim
                    aa$ = aa$ + "," + Str(Jdirect(i, j)) + "," + Str(Vdirect(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i

            GoTo Skip100
            sw.WriteLine("拘束節点データ")
            ad1(1) = "X拘束="
            ad1(2) = "Y拘束="
            ad1(3) = "Z拘束="
            ad1(4) = "θx拘束="
            ad1(5) = "θy拘束="
            ad1(6) = "θz拘束="
            ad2(1) = "δX="
            ad2(2) = "δY="
            ad2(3) = "δZ="
            ad2(4) = "θx="
            ad2(5) = "θy="
            ad2(6) = "θz="
            For i = 1 To Nbound
                aa$ = "拘束番号=" + "," + Str(i) + "," + "節点番号=" + "," + Str(Nrest(i))
                sw.WriteLine(aa$)
                j1 = Jdirect(i, 1)
                b1 = Vdirect(i, 1)
                j2 = Jdirect(i, 2)
                b2 = Vdirect(i, 2)
                j3 = Jdirect(i, 3)
                b3 = Vdirect(i, 3)
                aa$ = ad1(1) + "," + Str(j1) + "," + ad2(1) + "," + Str(b1)
                aa$ = aa$ + "," + ad1(2) + "," + Str(j2) + "," + ad2(2) + "," + Str(b2)
                aa$ = aa$ + "," + ad1(3) + "," + Str(j3) + "," + ad2(3) + "," + Str(b3)
                sw.WriteLine(aa$)
                If Ntype > 1 Then
                    j1 = Jdirect(i, 4)
                    b1 = Vdirect(i, 4)
                    j2 = Jdirect(i, 5)
                    b2 = Vdirect(i, 5)
                    j3 = Jdirect(i, 6)
                    b3 = Vdirect(i, 6)
                    aa$ = ad1(4) + "," + Str(j1) + "," + ad2(4) + "," + Str(b1)
                    aa$ = aa$ + "," + ad1(5) + "," + Str(j2) + "," + ad2(5) + "," + Str(b2)
                    aa$ = aa$ + "," + ad1(6) + "," + Str(j3) + "," + ad2(6) + "," + Str(b3)
                    sw.WriteLine(aa$)
                End If
            Next i
Skip100:
        End If

        aa$ = "材料データ (材料No.､E､ν、γ、A、ＩyB、ＩzB、K、    Ｚy1、Ｚy2、"
        aa$ = aa$ + "Ｚz1、Ｚz2、Ｃsy、Ｃsz、Ｚp）"
        sw.WriteLine(aa$)
        For im = 1 To Nmat
            aa$ = Str$(im)
            For j = 1 To Mprop
                If j = 8 Then
                    aa$ = aa$ + ",      " + Str$(DataMat(im, j))
                Else
                    aa$ = aa$ + ",  " + Str$(DataMat(im, j))
                End If
            Next j
            sw.WriteLine(aa$)
        Next im

        If Nforce > 0 Then
            If Ntype = 1 Then
                aa$ = "荷重節点 (荷重番号、節点番号、Ｆx、Ｆy、Ｆz）"
            Else
                aa$ = "荷重節点 (荷重番号、節点番号、Ｆx、Ｆy、Ｆz、Ｍx、Ｍy、Ｍz）"
            End If
            sw.WriteLine(aa$)
            For ib = 1 To Nforce
                aa$ = Str$(ib) + ",  " + Str$(KPforce(ib))
                For id = 1 To Ndim
                    aa$ = aa$ + ",  " + Str$(Force(ib, id))
                Next id
                sw.WriteLine(aa$)
            Next ib
        End If

        If Nsurf > 0 Then
            aa$ = "表面分布荷重（荷重番号、要素番号、"
            aa$ = aa$ + "部材Y､Z軸方向垂直分布荷重、接線分布荷重）"
            sw.WriteLine(aa$)
            For ib = 1 To Nsurf
                aa$ = Str$(ib)
                aa$ = aa$ + ",  " + Str$(KEsurf(ib))
                aa$ = aa$ + ",  " + Str$(PnormYB(ib))
                aa$ = aa$ + ",  " + Str$(PnormZB(ib))
                aa$ = aa$ + ",  " + Str$(Ptang(ib))
                sw.WriteLine(aa$)
            Next ib
        End If

        If Nspring > 0 Then
            aa$ = "バネ支持データ（No.、節点番号1、節点番号2、"
            aa$ = aa$ + "Ｋx、Ｆsx、Ｋy、Ｆsy、Ｋz、Ｆsz）"
            sw.WriteLine(aa$)
            For i = 1 To Nspring
                aa$ = Str$(i) + ",  " + Str$(KPspring(i, 1)) + ",  " + Str$(KPspring(i, 2))
                For j = 1 To 3
                    aa$ = aa$ + ",  " + Str$(Cspring(i, j))
                    aa$ = aa$ + ",  " + Str$(Fspring(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If KindGrav > 0 Then
            aa$ = "自重のX､Y､Z軸方向の倍率"
            sw.WriteLine(aa$)
            aa$ = Str$(GravMulti(1))
            For i = 2 To 3
                aa$ = aa$ + ",  " + Str$(GravMulti(i))
            Next i
            sw.WriteLine(aa$)
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Function DispMax(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j, k As Integer
        Dim b1, dmax As Single
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = NNelem(ie, j)
                    For k = 1 To 3
                        b1 = Math.Abs(tdb(i, k))
                        If b1 > dmax Then
                            dmax = b1
                        End If
                    Next k
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Sub DispRateSet(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim rlen, rlen2 As Single

        Call AxisCheck(KE1, KE2, rlen, Xdata, Ydata, Zdata)
        '座標の最小・最大値のﾁｪｯｸ

        rlen2 = DispMax(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Private Sub DistComp(ByRef ic As Integer, ByVal k1 As Integer, ByVal k2 As Integer)
        '距離比較
        'ic･･･距離区分（0:近い、1:遠い）
        'k1･･･基準要素番号
        'k2･･･比較要素番号
        Dim i1, i2, j1, j2 As Integer
        Dim alphab, b1, b2, b3, b4, beta, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        ic = -1  '仮設定
        i1 = NNelem(k1, 1)
        i2 = NNelem(k1, 2)
        xbuf(1) = Xscreen(i1)
        ybuf(1) = Yscreen(i1)
        xbuf(2) = Xscreen(i2)
        ybuf(2) = Yscreen(i2)
        b1 = xbuf(1) - xbuf(2)
        b2 = ybuf(1) - ybuf(2)
        b3 = b1 * b1 + b2 * b2

        j1 = NNelem(k2, 1)
        j2 = NNelem(k2, 2)
        xbufc(1) = Xscreen(j1)
        ybufc(1) = Yscreen(j1)
        xbufc(2) = Xscreen(j2)
        ybufc(2) = Yscreen(j2)
        b1 = xbufc(1) - xbufc(2)
        b2 = ybufc(1) - ybufc(2)
        b4 = b1 * b1 + b2 * b2
        If b4 > b3 Then
            b3 = b4
        End If
        b3 = 0.0001 * Math.Sqrt(b3)

        Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
        '2要素の交点計算

        If alphab >= 0 Then
            b1 = 1 - alphab
            xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
            yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
            zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            b1 = 1 - beta
            xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
            yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
            zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
            distc = xb * xb + yb * yb + zb * zb
            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
        ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
        ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub EDatSet(ByVal neb As Integer, ByVal ne As Integer)
        '要素ﾃﾞｰﾀの保存変更
        'neb･･･旧要素数
        'ne･･･新要素数（増加）
        Dim ie, j As Integer
        Dim kbuf(neb), nnbuf(neb, 2), mbuf(neb) As Integer
        Dim anglebuf(neb) As Single
        For ie = 1 To neb
            kbuf(ie) = KindElem(ie)
            mbuf(ie) = NumMat(ie)
            For j = 1 To 2
                nnbuf(ie, j) = NNelem(ie, j)
            Next j
            anglebuf(ie) = AngleYBar(ie)
        Next ie
        ReDim KindElem(ne), NNelem(ne, 2), NumMat(ne)
        ReDim AngleYBar(ne)
        For ie = 1 To neb
            KindElem(ie) = kbuf(ie)
            NumMat(ie) = mbuf(ie)
            For j = 1 To 2
                NNelem(ie, j) = nnbuf(ie, j)
            Next j
            AngleYBar(ie) = anglebuf(ie)
        Next ie

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemDeleteB3D(ByVal nx As Integer, ByRef kdels() As Integer)
        '要素の削除B3D
        'nx･･･要素削除の個数
        'kdels()･･･削除個数分の要素番号
        Dim i, ib, j, j2, k, num As Integer
        Dim kbuf(Npoint), kbuf2(Nelem), kdelb(Nelem), nbuf(Npoint) As Integer
        Dim xbuf(Npoint), ybuf(Npoint), zbuf(Npoint) As Single

        For i = 1 To nx
            kdelb(kdels(i)) = 1
        Next i
        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i

        For i = 1 To Nelem
            If kdelb(i) = 0 Then
                For j = 1 To 2
                    ib = NNelem(i, j)
                    kbuf(ib) = 1
                Next j
            End If
        Next i
        num = 0
        For i = 1 To Npoint
            If kbuf(i) > 0 Then
                num = num + 1
                nbuf(i) = num
                kbuf(num) = i
            End If
        Next i
        Npoint = num
        For i = 1 To Npoint
            Xdata(i) = xbuf(kbuf(i))
            Ydata(i) = ybuf(kbuf(i))
            Zdata(i) = zbuf(kbuf(i))
        Next i

        num = 0
        For i = 1 To Nelem
            If kdelb(i) = 0 Then
                num = num + 1
                NumMat(num) = NumMat(i)
                For j = 1 To 2
                    k = NNelem(i, j)
                    NNelem(num, j) = nbuf(k)
                Next j
                If Ntype = 3 Then
                    KindElem(num) = KindElem(i)
                End If
                kbuf2(i) = num
            End If
        Next i
        Nelem = num

        If Nbound > 0 Then
            num = 0
            For i = 1 To Nbound
                k = Nrest(i)
                If nbuf(k) > 0 Then
                    num = num + 1
                    Nrest(num) = nbuf(k)
                    If Ntype = 1 Then
                        j2 = 3
                    Else
                        j2 = 6
                    End If
                    For j = 1 To j2
                        Jdirect(num, j) = Jdirect(i, j)
                        Vdirect(num, j) = Vdirect(i, j)
                    Next j
                End If
            Next i
            Nbound = num
        End If

        If Nforce > 0 Then
            num = 0
            For i = 1 To Nforce
                k = KPforce(i)
                If nbuf(k) > 0 Then
                    num = num + 1
                    KPforce(num) = nbuf(k)
                    For j = 1 To Ndim
                        Force(num, j) = Force(i, j)
                    Next j
                End If
            Next i
            Nforce = num
        End If

        If Nsurf > 0 Then
            num = 0
            For i = 1 To Nsurf
                k = KEsurf(i)
                If kbuf2(k) > 0 Then
                    num = num + 1
                    KEsurf(num) = kbuf2(k)
                    PnormYB(num) = PnormYB(i)
                    PnormZB(num) = PnormZB(i)
                    Ptang(num) = Ptang(i)
                End If
            Next i
            Nsurf = num
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xsb() As Single, ByRef ysb() As Single)
        '拡大作画する要素の探索
        '両節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        'xsb(),ysb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer

        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    b1 = xsb(ib) - Xmouse(1)
                    b2 = xsb(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = ysb(ib) - Ymouse(2)
                        b2 = ysb(ib) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 2 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub ElemSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画要素のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, j, nb As Integer

        NEdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                NEdraw = NEdraw + 1
            End If
        Next ie
        ReDim KEorder(NEdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                nb = nb + 1
                KEorder(nb) = ie
            End If
        Next ie

        For ii = 1 To NEdraw - 1
            i1 = KEorder(ii)
            For j = ii + 1 To NEdraw
                i2 = KEorder(j)
                Call DistComp(ic, i1, i2)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KEorder(j) = i1
                    i1 = i2
                End If
            Next j
        Next ii

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByVal abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf$(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub MDatSet2(ByVal nm1 As Integer, ByVal nm2 As Integer)
        '材料ﾃﾞｰﾀの保存変更2
        'nm1･･･初期の材料数
        'nm2･･･変更後の材料数（同じか増加）
        Dim im, j As Integer
        Dim dmbuf(nm1, Mprop) As Single
        For im = 1 To nm1
            For j = 1 To Mprop
                dmbuf(im, j) = DataMat(im, j)
            Next j
        Next im
        ReDim DataMat(nm2, Mprop)
        For im = 1 To nm1
            For j = 1 To Mprop
                DataMat(im, j) = dmbuf(im, j)
            Next j
        Next im

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseCalcS(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            xb = (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
        Else
            xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale + YaxisMin
        End If

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
        ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single, _
        ByRef xmb() As Single, ByRef ymb() As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        'xmb(),ymb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = xmb(ib) - xb
                        b2 = ymb(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NvarPDSet()
        '元ﾃﾞｰﾀ節点の変数の数を設定
        Dim i, ib, ii, ip, j, k, nb As Integer

        For i = 1 To Npoint
            NvarPD(i) = 3
        Next i
        For i = 1 To Nelem
            If KindElem(i) > 1 Then
                For j = 1 To 2
                    k = NNelem(i, j)
                    NvarPD(k) = 6
                Next j
            End If
        Next i

        '行列の変数番号の対応設定
        ii = 0
        For ip = 1 To Npoint
            nb = NvarPD(ip)
            ib = 0
            LrD(ip) = ii + 1
            For j = 1 To nb
                ii = ii + 1
                ib = ib + 1
            Next j
        Next ip

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3Menu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            MultiColor = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3Menu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.WriteLine(MultiColor)
        sw.Close()

    End Sub

    Sub PDatSet(ByVal npb As Integer, ByVal np As Integer)
        '節点ﾃﾞｰﾀの保存変更
        'npb･･･旧節点数
        'np･･･新節点数（増加）
        Dim ip As Integer
        Dim xbuf(npb), ybuf(npb), zbuf(npb) As Single
        For ip = 1 To npb
            xbuf(ip) = Xdata(ip)
            ybuf(ip) = Ydata(ip)
            zbuf(ip) = Zdata(ip)
        Next ip
        ReDim Xdata(np), Ydata(np), Zdata(np)
        For ip = 1 To npb
            Xdata(ip) = xbuf(ip)
            Ydata(ip) = ybuf(ip)
            Zdata(ip) = zbuf(ip)
        Next ip

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub SpringCheck2()
        '節点間バネのﾁｪｯｸ
        Dim i, j As Integer
        For i = 1 To Nspring
            If KPspring(i, 2) > 0 Then
                For j = 2 To 3
                    Cspring(i, j) = 0
                    Fspring(i, j) = 0
                Next j
            End If
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TimCal(ByRef ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalA(ByVal xs As Single, ByVal ys As Single, _
    ByVal zs As Single, ByVal xr As Single, ByVal yr As Single, _
    ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '先にZ,Y,X軸回りの回転、最後に座標軸方向の平行移動
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TMatrixCos(ByVal thetax As Single, ByVal thetay As Single, _
        ByVal thetaz As Single)
        '方向余弦用変換ﾏﾄﾘｯｸｽの計算
        'thetax･･･X軸回りの回転角ﾗｼﾞｱﾝ（第1ｽﾃｯﾌﾟ）
        'thetay･･･Y軸回りの回転角ﾗｼﾞｱﾝ（第2ｽﾃｯﾌﾟ）
        'thetaz･･･Z軸回りの回転角ﾗｼﾞｱﾝ（第3ｽﾃｯﾌﾟ）
        Dim i, j, k As Integer
        Dim b1, b2 As Single
        Dim buf(4, 4), buf1(4, 4), buf2(4, 4), buf3(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf3(1, 1) = 1
        buf3(2, 2) = b1
        buf3(2, 3) = -b2
        buf3(3, 2) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf2(1, 1) = b1
        buf2(1, 3) = b2
        buf2(2, 2) = 1
        buf2(3, 1) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf2(i, k) * buf3(k, j)
                Next k
                buf(i, j) = b1
            Next j
        Next i

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf1(1, 1) = b1
        buf1(1, 2) = -b2
        buf1(2, 1) = b2
        buf1(2, 2) = b1
        buf1(3, 3) = 1
        buf1(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3UFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3UFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub VCosCalc(ByRef kh As Integer)
        '断面ＹB軸の方向余弦計算
        'kh･･･ｴﾗｰ有無（0:なし、1:有り）
        Dim aa$
        Dim i, ic, ie, j, k1, k2 As Integer
        Dim alpha, b1, hpai, rlen1, rlenb As Single
        Dim thetax, thetay, thetaz As Single
        Dim buf(3), vec(4) As Single

        hpai = 1.570796
        alpha = Pai / 180
        kh = 0
        For ie = 1 To Nelem
            k1 = NNelem(ie, 1)
            k2 = NNelem(ie, 2)
            buf(1) = Xdata(k2) - Xdata(k1)
            buf(2) = Ydata(k2) - Ydata(k1)
            buf(3) = Zdata(k2) - Zdata(k1)

            'ﾃﾞｰﾀ異常のﾁｪｯｸ
            If buf(1) = 0 And buf(2) = 0 And buf(3) = 0 Then
                kh = 1
                aa$ = "要素" + Str$(ie) + "の両端節点が重なっています！"
                MsgBox(aa$, 0, "データエラー")
                Exit Sub
            End If

            thetax = -AngleYBar(ie) * alpha     '第1ｽﾃｯﾌﾟ
            If buf(3) < 0 Then  '要素を上向き方向に変換
                thetax = -thetax
                For j = 1 To 3
                    buf(j) = -buf(j)
                Next j
            End If
            '第2ｽﾃｯﾌﾟ
            ic = 0
            If buf(3) = 0 Then  '水平の場合
                thetay = 0
            ElseIf buf(1) = 0 And buf(2) = 0 Then   '垂直の場合
                thetay = hpai
                ic = 1
                thetaz = 0
            Else
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + buf(j) * buf(j)
                Next j
                rlenb = Math.Sqrt(b1)     '要素長さ
                b1 = 0
                For j = 1 To 2
                    b1 = b1 + buf(j) * buf(j)
                Next j
                rlen1 = Math.Sqrt(b1)     'XY平面への投影長さ
                thetay = -Math.Atan(buf(3) / rlen1)   'X軸上での回転角
            End If
            If ic = 0 Then
                If buf(1) = 0 Then  'X座標が同じ場合
                    If buf(2) > 0 Then
                        thetaz = hpai
                    Else
                        thetaz = -hpai
                    End If
                ElseIf buf(2) = 0 Then  'Y座標が同じ場合
                    If buf(1) > 0 Then
                        thetaz = 0
                    Else
                        thetaz = Pai
                    End If
                Else
                    thetaz = Math.Atan(buf(2) / buf(1))
                    If buf(1) < 0 Then
                        thetaz = thetaz + Pai
                    End If
                End If
            End If
            Call TMatrixCos(thetax, thetay, thetaz)  '方向余弦用変換ﾏﾄﾘｯｸｽの計算
            vec(1) = 0
            vec(2) = 0
            vec(3) = 1  'Z軸ﾍﾞｸﾄﾙに取る
            vec(4) = 1
            For i = 1 To 3
                b1 = 0
                For j = 1 To 4
                    b1 = b1 + TransMat(i, j) * vec(j)
                Next j
                CosYBar(ie, i) = b1
            Next i
        Next ie
    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
